@@ -10,7 +10,7 @@
  *
  * This program has been updated to use the new PPS API v1 from RFC 2783
  *
- * Copyright (c) 1996 - 2000 by Ulrich Windl
+ * Copyright (c) 1996 - 2001 by Ulrich Windl
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -26,7 +26,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: enable_pps.c,v 1.38 2000/09/14 19:45:30 windl Exp $
+ * $Id: enable_pps.c,v 1.38.2.1 2001/04/14 15:15:56 windl Exp $
  */
 #include	<unistd.h>
 #include	<stdlib.h>
@@ -34,6 +34,7 @@
 #include	<ctype.h>
 #include	<string.h>
 #include	<errno.h>
+#include	<fcntl.h>
 #include	<sys/time.h>
 #include	<sys/utsname.h>
 
@@ -41,8 +42,6 @@
 #define	NANO	/* prefer the Nanokernel if available */
 #include	<sys/timex.h>
 
-#include	<sys/ppsclock.h>
-
 #include	<sys/ioctl.h>
 #include	<sys/timepps.h>
 #if defined(__GNU_LIBRARY__) && __GNU_LIBRARY__ >= 6	/* libc6 or glibc2 */
@@ -84,7 +83,7 @@
 #define	BITS_OFF(v, b)			((v) & ~(b))
 #define	BITS_ON(v, b)			((v) | (b))
 
-static	const char	id[] = "$Id: enable_pps.c,v 1.38 2000/09/14 19:45:30 windl Exp $";
+static	const char	id[] = "$Id: enable_pps.c,v 1.38.2.1 2001/04/14 15:15:56 windl Exp $";
 
 #define	BIT(n)	(1UL << (n))
 #define OPT_MONITOR		BIT(0)	/* perform monitor loop */
@@ -102,7 +101,7 @@ static	unsigned long	options = OPT_MONITOR | OPT_RESET;
 
 static	char		trigger	= '0';	/* off | assert | clear | both */
 
-static	const	int	fd	= 0;	/* file descriptor of port (stdin) */
+static	int		port_fd	= -1;	/* file descriptor of port */
 static	pps_handle_t	ppshandle;	/* PPS handle (stdin) */
 
 struct clock_info {			/* information about clock and conditions */
@@ -614,15 +613,15 @@ static	void	monitor_jitter(struct clock_info *cip)
 			continue;
 		}
 		update_clock_info(cip);
-		if ( cip->seq == last.seq )
+		if ( cip->dseq == 0 )
 			continue;
 		pps_status("monitor_jitter(): ", 0, cip);
 		if ( (options & OPT_JITTER_SILENT) == 0 ||
 		     (cip->tx.status & ALL_ERRORS) != 0 ) {
-			printf("event(%4lu:%4lu) jitter %9ld ns, offset %9ld ns",
+			printf("event(%4lu:%4lu) jitter %9ld ns, nsec %9ld",
 			       cip->info.assert_sequence,
 			       cip->info.clear_sequence,
-			       cip->jitter, cip->nano_offset);
+			       cip->jitter, cip->nsec);
 			if ( (cip->tx.status & STA_PPSJITTER) != 0 ||
 			     cip->jitter > 1000000 || cip->jitter < -1000000 )
 			{
@@ -659,19 +658,20 @@ static	void	usage(void)
 #ifdef MOD_PPSMAX
 		"\t-PX\tset PPS shiftmax to `X'\n"
 #endif
+		"\t-pX\tuse port (device) `X'\n"
 		"\t-S\tdon't adjust status in monitoring loop\n"
 		"\t-t{0|a|c|b}[h]\ttrigger for event: off|assert|clear|both\n"
 		"\t\t'h' = route to hardpps()\n"
 		);
 }
 
-/* input has to be redirected to the desired device! */
+/* Parse command line arguments and perform requested action */
 int	main(int argc, char *argv[])
 {
 	struct utsname	un;
-	int		stat = TIOCM_RTS;	/* RTS MODEM control line */
 	struct clock_info ci;
 	pps_params_t	parm;
+	int		capabilities;
 	int		kc_edge = 0;
 	int		ch;
 
@@ -692,19 +692,8 @@ int	main(int argc, char *argv[])
 	       );
 	uname(&un);
 	printf(". Now running %s %s)\n", un.sysname, un.release);
-	if ( time_pps_create(fd, &ppshandle) )
-	{
-		perror("time_pps_create()");
-		return(1);
-	}
-	if ( time_pps_getcap(ppshandle, &parm.mode) )
-	{
-		perror("time_pps_getcap()");
-		return(1);
-	}
-	printf("PPS API capabilities are 0x%x\n", parm.mode);
 	parm.mode = PPS_TSFMT_TSPEC;	/* set default flags */
-	while ( (ch = getopt(argc, argv, "CE:eFJjkMNOo:P:St:")) != -1 )
+	while ( (ch = getopt(argc, argv, "CE:eFJjkMNOo:P:p:St:")) != -1 )
 	{
 		switch ( ch )
 		{
@@ -781,6 +770,23 @@ int	main(int argc, char *argv[])
 			verbose_adjtimex("MOD_PPSMAX", 0, &ci);
 			break;
 #endif
+		case 'p':
+			if ( (port_fd = open(optarg, O_RDWR, 0)) == -1 )
+			{
+				fprintf(stderr, "open(%s): %s\n",
+					optarg, strerror(errno));
+				exit(1);
+			}
+			else
+			{
+				int	mstat;	/* MODEM line status */
+				
+				mstat = TIOCM_RTS;
+				/* set DTR on, RTS off to provide power for
+				   level converter */
+				ioctl(port_fd, TIOCMBIC, &mstat);
+			}
+			break;
 		case 'S':	/* don't adjust time status */
 			options = BITS_ON(options, OPT_NO_STATUS);
 			break;
@@ -815,7 +821,25 @@ int	main(int argc, char *argv[])
 		exit(1);
 	}
 
-	ioctl(fd, TIOCMBIC, &stat);	/* power-on DCF77 receiver */
+	if ( port_fd == -1 )
+	{
+		fprintf(stderr, "You must specify the port to be used!\n");
+		usage();
+		exit(1);
+	}
+
+	if ( time_pps_create(port_fd, &ppshandle) )
+	{
+		perror("time_pps_create()");
+		return(1);
+	}
+	if ( time_pps_getcap(ppshandle, &capabilities) )
+	{
+		perror("time_pps_getcap()");
+		return(1);
+	}
+	printf("PPS API capabilities for handle %d are 0x%x\n",
+	       ppshandle, capabilities);
 	switch ( trigger )
 	{
 	case 'A':
@@ -846,7 +870,7 @@ int	main(int argc, char *argv[])
 	else
 	{
 		fprintf(stderr,
-			"PPS API v%d set up (handle %d, mode %#04x)\n",
+			"PPS API v%d set up on handle %d, mode %#04x\n",
 			parm.api_version, ppshandle, parm.mode);
 		if ( kc_edge != 0 )
 		{
@@ -862,8 +886,8 @@ int	main(int argc, char *argv[])
 			{
 				fprintf(stderr,
 					"PPS API kernel consumer HARDPPS bound"
-					" to edge %#04x\n",
-					kc_edge);
+					" to edge %#04x on handle %d\n",
+					kc_edge, ppshandle);
 			}
 		}
 	}
